@@ -17,13 +17,11 @@
 # with logilab-database. If not, see <http://www.gnu.org/licenses/>.
 """logilab.database packaging information."""
 
-import sys
-
-distname = 'logilab-database'
-modname = 'database'
-numversion = (1, 14, 1)
-version = '.'.join([str(num) for num in numversion])
-license = 'LGPL'
+distname = "logilab-database"
+modname = "database"
+numversion = (1, 18, 0)
+version = ".".join([str(num) for num in numversion])
+license = "LGPL"
 
 author = "Logilab"
 author_email = "contact@logilab.fr"
@@ -33,24 +31,20 @@ description = "true unified database access"
 web = "http://www.logilab.org/project/%s" % distname
 mailinglist = "mailto://python-projects@lists.logilab.org"
 
-subpackage_of = 'logilab'
+subpackage_of = "logilab"
 
 install_requires = [
-    'setuptools',
-    'logilab-common >= 0.63.2',
-    'six >= 1.4.0',
-    'Yapps2',
-    ]
+    "setuptools",
+    "logilab-common >= 0.63.2",
+    "Yapps2",
+    "python-dateutil",
+]
 
-tests_require = [
-    'psycopg2',
-    ]
+tests_require = ["psycopg2"]
 
-if sys.version_info[0] == 2:
-    tests_require.append('MySQL-python')
-
-classifiers = ["Topic :: Database",
-               "Programming Language :: Python",
-               "Programming Language :: Python :: 2",
-               "Programming Language :: Python :: 3",
-               ]
+classifiers = [
+    "Topic :: Database",
+    "Programming Language :: Python",
+    "Programming Language :: Python :: 3",
+    "Programming Language :: Python :: 3 :: Only",
+]
